package com.kickstarter.viewmodels

import android.util.Pair
import androidx.annotation.NonNull
import com.kickstarter.libs.Environment
import com.kickstarter.libs.FragmentViewModel
import com.kickstarter.libs.rx.transformers.Transformers.combineLatestPair
import com.kickstarter.libs.rx.transformers.Transformers.takeWhen
import com.kickstarter.libs.utils.ObjectUtils
import com.kickstarter.libs.utils.RewardUtils
import com.kickstarter.libs.utils.extensions.isBacked
import com.kickstarter.mock.factories.RewardFactory
import com.kickstarter.models.Backing
import com.kickstarter.models.Project
import com.kickstarter.models.Reward
import com.kickstarter.ui.data.PledgeData
import com.kickstarter.ui.data.PledgeFlowContext
import com.kickstarter.ui.data.PledgeReason
import com.kickstarter.ui.data.ProjectData
import com.kickstarter.ui.fragments.RewardsFragment
import com.kickstarter.viewmodels.usecases.ShowPledgeFragmentUseCase
import rx.Observable
import rx.subjects.BehaviorSubject
import rx.subjects.PublishSubject
import java.util.Locale

class RewardsFragmentViewModel {
    interface Inputs {
        /** Configure with current [ProjectData]. */
        fun configureWith(projectData: ProjectData)

        /** Call when a reward is clicked.  */
        fun rewardClicked(reward: Reward)

        /** Call when the Alert button has been pressed  */
        fun alertButtonPressed()
    }

    interface Outputs {
        /**  Emits the position of the backed reward. */
        fun backedRewardPosition(): Observable<Int>

        /** Emits the current [ProjectData]. */
        fun projectData(): Observable<ProjectData>

        /** Emits the count of the current project's rewards. */
        fun rewardsCount(): Observable<Int>

        /** Emits when we should show the [com.kickstarter.ui.fragments.PledgeFragment].  */
        fun showPledgeFragment(): Observable<Triple<PledgeData, PledgeReason, Boolean>>

        /** Emits when we should show the [com.kickstarter.ui.fragments.BackingAddOnsFragment].  */
        fun showAddOnsFragment(): Observable<Pair<PledgeData, PledgeReason>>

        /** Emits if we have to show the alert in case any AddOns selection could be lost. */
        fun showAlert(): Observable<Pair<PledgeData, PledgeReason>>
    }

    class ViewModel(@NonNull val environment: Environment) : FragmentViewModel<RewardsFragment>(environment), Inputs, Outputs {

        private val projectDataInput = PublishSubject.create<ProjectData>()
        private val rewardClicked = PublishSubject.create<Pair<Reward, Boolean>>()
        private val alertButtonPressed = PublishSubject.create<Void>()

        private val backedRewardPosition = PublishSubject.create<Int>()
        private val projectData = BehaviorSubject.create<ProjectData>()
        private val rewardsCount = BehaviorSubject.create<Int>()
        private val pledgeData = PublishSubject.create<Pair<PledgeData, PledgeReason>>()
        private val showPledgeFragment = PublishSubject.create<Triple<PledgeData, PledgeReason, Boolean>>()
        private val showAddOnsFragment = PublishSubject.create<Pair<PledgeData, PledgeReason>>()
        private val showAlert = PublishSubject.create<Pair<PledgeData, PledgeReason>>()

        // - Environment Objects
        private val currentUser = requireNotNull(this.environment.currentUser()?.observable())
        private val optimizely = requireNotNull(this.environment.optimizely())

        val inputs: Inputs = this
        val outputs: Outputs = this

        init {

            this.isExpanded
                .filter { it }
                .compose(combineLatestPair(this.projectDataInput))
                .map { it.second }
                .compose(bindToLifecycle())
                .subscribe { this.analyticEvents.trackRewardsCarouselViewed(it) }

            this.projectDataInput
                .map { filterOutNotStartedRewards(it) }
                .compose(bindToLifecycle())
                .subscribe(this.projectData)

            val project = this.projectData
                .map { it.project() }

            project
                .filter { it.isBacking() }
                .map { indexOfBackedReward(it) }
                .distinctUntilChanged()
                .compose(bindToLifecycle())
                .subscribe(this.backedRewardPosition)

            val backedReward = project
                .map { it.backing()?.let { backing -> getReward(backing) } }
                .filter { ObjectUtils.isNotNull(it) }
                .map { requireNotNull(it) }

            val defaultRewardClicked = Pair(Reward.builder().id(0L).minimum(0.0).build(), false)

            Observable
                .combineLatest(this.rewardClicked.startWith(defaultRewardClicked), this.projectDataInput) { rewardPair, projectData ->
                    if (!rewardPair.second) {
                        return@combineLatest null
                    } else {
                        return@combineLatest pledgeDataAndPledgeReason(projectData, rewardPair.first)
                    }
                }
                .filter { ObjectUtils.isNotNull(it) }
                .map { requireNotNull(it) }
                .compose(bindToLifecycle())
                .subscribe {
                    val pledgeAndData = it
                    val newRw = it.first.reward()
                    val reason = it.second

                    when (reason) {
                        PledgeReason.PLEDGE -> {
                            if (newRw.hasAddons())
                                this.showAddOnsFragment.onNext(pledgeAndData)
                            else
                                this.pledgeData.onNext(pledgeAndData)
                        }
                        else -> {}
                    }
                    this.rewardClicked.onNext(defaultRewardClicked)
                }

            Observable
                .combineLatest(this.rewardClicked.startWith(defaultRewardClicked), this.projectDataInput, backedReward) { rewardPair, projectData, backedReward ->
                    if (!rewardPair.second) {
                        return@combineLatest null
                    } else {
                        return@combineLatest Pair(pledgeDataAndPledgeReason(projectData, rewardPair.first), backedReward)
                    }
                }
                .filter { ObjectUtils.isNotNull(it) }
                .map { requireNotNull(it) }
                .compose(bindToLifecycle())
                .subscribe {
                    val pledgeAndData = it.first
                    val newRw = it.first.first.reward()
                    val prevRw = it.second
                    val reason = it.first.second

                    when (reason) {
                        PledgeReason.UPDATE_REWARD -> {
                            if (prevRw.hasAddons() && !newRw.hasAddons())
                                this.showAlert.onNext(pledgeAndData)

                            if (!prevRw.hasAddons() && !newRw.hasAddons())
                                this.pledgeData.onNext(pledgeAndData)

                            if (prevRw.hasAddons() && newRw.hasAddons()) {
                                if (differentShippingTypes(prevRw, newRw)) this.showAlert.onNext(it.first)
                                else this.showAddOnsFragment.onNext(pledgeAndData)
                            }

                            if (!prevRw.hasAddons() && newRw.hasAddons()) {
                                this.showAddOnsFragment.onNext(pledgeAndData)
                            }
                        }
                        else -> {}
                    }
                    this.rewardClicked.onNext(defaultRewardClicked)
                }

            project
                .map { it.rewards()?.size ?: 0 }
                .compose(bindToLifecycle())
                .subscribe(this.rewardsCount)

            this.showAlert
                .compose<Pair<PledgeData, PledgeReason>>(takeWhen(alertButtonPressed))
                .compose(bindToLifecycle())
                .subscribe {
                    if (it.first.reward().hasAddons())
                        this.showAddOnsFragment.onNext(it)
                    else this.pledgeData.onNext(it)
                }

            ShowPledgeFragmentUseCase(this.pledgeData)
                .data(this.currentUser, this.optimizely)
                .compose(bindToLifecycle())
                .subscribe {
                    this.showPledgeFragment.onNext(it)
                }
        }

        private fun filterOutNotStartedRewards(pData: ProjectData): ProjectData {
            val rewards = pData.project().rewards()?.filter { RewardUtils.hasStarted(it) }
            val modifiedProject = pData.project().toBuilder().rewards(rewards).build()
            return pData.toBuilder()
                .project(modifiedProject)
                .build()
        }

        private fun getReward(backingObj: Backing): Reward {
            return backingObj.reward()?.let { rw ->
                if (backingObj.addOns().isNullOrEmpty()) rw
                else rw.toBuilder().hasAddons(true).build()
            } ?: RewardFactory.noReward()
        }

        private fun differentShippingTypes(newRW: Reward, backedRW: Reward): Boolean {
            return if (newRW.id() == backedRW.id()) false
            else {
                newRW.shippingType()?.toLowerCase(Locale.getDefault()) ?: "" != backedRW.shippingType()?.toLowerCase(Locale.getDefault()) ?: ""
            }
        }

        private fun pledgeDataAndPledgeReason(projectData: ProjectData, reward: Reward): Pair<PledgeData, PledgeReason> {
            val pledgeReason = if (projectData.project().isBacking()) PledgeReason.UPDATE_REWARD else PledgeReason.PLEDGE
            val pledgeData = PledgeData.with(PledgeFlowContext.forPledgeReason(pledgeReason), projectData, reward)
            return Pair(pledgeData, pledgeReason)
        }

        private fun indexOfBackedReward(project: Project): Int {
            project.rewards()?.run {
                for ((index, reward) in withIndex()) {
                    if (project.backing()?.isBacked(reward) == true) {
                        return index
                    }
                }
            }

            return 0
        }

        override fun configureWith(projectData: ProjectData) {
            this.projectDataInput.onNext(projectData)
        }

        override fun rewardClicked(reward: Reward) {
            this.rewardClicked.onNext(Pair(reward, true))
        }

        override fun alertButtonPressed() = this.alertButtonPressed.onNext(null)

        @NonNull
        override fun backedRewardPosition(): Observable<Int> = this.backedRewardPosition

        @NonNull
        override fun projectData(): Observable<ProjectData> = this.projectData

        @NonNull
        override fun rewardsCount(): Observable<Int> = this.rewardsCount

        @NonNull
        override fun showPledgeFragment(): Observable<Triple<PledgeData, PledgeReason, Boolean>> = this.showPledgeFragment

        @NonNull
        override fun showAddOnsFragment(): Observable<Pair<PledgeData, PledgeReason>> = this.showAddOnsFragment

        @NonNull
        override fun showAlert(): Observable<Pair<PledgeData, PledgeReason>> = this.showAlert
    }
}
